# frozen_string_literal: true

# app/hermes/models/concerns/status.rb
# app/controllers/concerns/test_concern.rb

module Hermes
  module Concerns
    # Handles statuses relationships
    module Status
      extend ActiveSupport::Concern

      included do
        self.class_variable_set(:@@status_names, [])
      end
      
      def check_status_changed
        class_name = self.class.name.demodulize.underscore

        status_names.each do |n|
          if send("#{n}_changed?")
            prev_status = Hermes::Status.where("#{class_name}_id": id, status_label: n).order(created_at: :desc).limit(1).first
            if !prev_status || prev_status.title != send("#{n}")
              
              new_status = Hermes::Status.create(
                object_id: id,
                status_label: n,
                title: send("#{n}"),
                closing: prev_status ? prev_status.id : nil
              )

              send("#{n}_id=", new_status.id.to_s)
              send("#{n}_updated_at=", new_status.created_at)
              send("old_#{n}=", prev_status ? prev_status.title : nil)
              save
            end
          end
        end
      end

      def status_names
        self.class.class_variable_get(:@@status_names)
      end
      
      module ClassMethods
        def add_status(type = :single, label_name = "status", default = nil)
          self.class_variable_set(:@@status_names, self.class_variable_get(:@@status_names).push(label_name))

          case type
          when :single
            field "#{label_name}", type: String, default: default
            field "#{label_name}_id", type: BSON::ObjectId
            field "#{label_name}_updated_at", type: DateTime
            field "old_#{label_name}", type: String
          when :multiple
            field "#{label_name}", type: Array
          else
            raise ArgumentError, "To add a status the type should be :single or :multiple. #{type} isn't a valid type"
          end
        end
      end
    end
  end
end
